@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 A striplog is a sequence of intervals.
 
@@ -18,12 +17,15 @@ from copy import deepcopy
 import numpy as np
 import matplotlib as mpl
 import matplotlib.pyplot as plt
+import requests
+import json
 
 from .interval import Interval, IntervalError
 from .component import Component
 from .legend import Legend
 from .canstrat import parse_canstrat
 from .markov import Markov_chain
+from .lexicon import Lexicon
 from . import utils
 from . import templates
 
@@ -415,8 +417,8 @@ class Striplog(object):
 
         return list_of_Intervals
 
-    @classmethod
-    def _clean_longitudinal_data(cls, data, null=None):
+    @staticmethod
+    def _clean_longitudinal_data(data, null=None):
         """
         Private function. Make sure we have what we need to make a striplog.
         """
@@ -502,6 +504,12 @@ class Striplog(object):
         Returns:
             list.
         """
+        if not lexicon:
+            lexicon = Lexicon.default()
+            with warnings.catch_warnings():
+                warnings.simplefilter("module")
+                w = 'No lexicon provided, using the default.'
+                warnings.warn(w)
 
         include = include or {}
         exclude = exclude or {}
@@ -567,11 +575,18 @@ class Striplog(object):
                         if v is not None:
                             d[k] = v  # It's data
                 comp = [Component(c)] if c else None
-                this = Interval(**{'top': top,
-                                   'base': base,
-                                   'description': descr,
-                                   'data': d,
-                                   'components': comp})
+                if comp:
+                    this = Interval(**{'top': top,
+                                    'base': base,
+                                    'description': descr,
+                                    'data': d,
+                                    'components': comp})
+                elif not comp:
+                    this = Interval(**{'top': top,
+                                    'base': base,
+                                    'data': d,
+                                    'description': descr,
+                                    'lexicon': lexicon})
             else:
                 this = Interval(**{'top': top,
                                    'base': base,
@@ -677,6 +692,25 @@ class Striplog(object):
 
         return cls(list_of_Intervals, source=source)
 
+    @classmethod
+    def from_dict(cls, dictionary):
+        """
+        Take a dictionary of the form name:depth and return a striplog of
+        complete intervals.
+        """
+        d_sorted = sorted(dictionary.items(), key=lambda i: i[1])
+        names = [i[0] for i in d_sorted]
+        tops_ = [i[1] for i in d_sorted]
+        bases_ = tops_[1:] + [tops_[-1]+1]
+        comps_ = [Component({'formation': name}) for name in names]
+
+        list_of_Intervals = []
+        for top, base, comp in zip(tops_, bases_, comps_):
+            iv = Interval(top=top, base=base, components=[comp])
+            list_of_Intervals.append(iv)
+
+        return cls(list_of_Intervals)
+
     @classmethod
     def from_descriptions(cls, text,
                           lexicon=None,
@@ -856,6 +890,9 @@ class Striplog(object):
                                                       basis,
                                                       components)
 
+        list_of_Intervals = [iv for iv in list_of_Intervals
+                             if isinstance(iv.primary, Component)]
+
         return cls(list_of_Intervals, source="Image")
 
     @classmethod
@@ -951,6 +988,13 @@ class Striplog(object):
         Returns:
             Striplog: The ``striplog`` object.
         """
+        # Try to deal with the log potentially being a welly.Curve, pd.Series,
+        # or pd.DataFrame. Squeeze down to 1D in case it's a column vector.
+        try:
+            log_arr = np.squeeze(log.values)
+        except AttributeError:
+            log_arr = np.squeeze(np.asanyarray(log))
+
         if (components is None) and (legend is None) and (field is None):
             m = 'You must provide a list of components and legend, or a field.'
             raise StriplogError(m)
@@ -975,18 +1019,18 @@ class Striplog(object):
             except TypeError:
                 n = 1
             if len(components) < n+1:
-                m = 'For n cutoffs, you need to provide at least'
+                m = 'For n cutoffs, you need to provide at least '
                 m += 'n+1 components.'
                 raise StriplogError(m)
 
             # Digitize.
             try:  # To use cutoff as a list.
-                a = np.digitize(log, cutoff, right)
+                a = np.digitize(log_arr, cutoff, right)
             except ValueError:  # It's just a number.
-                a = np.digitize(log, [cutoff], right)
+                a = np.digitize(log_arr, [cutoff], right)
 
         else:
-            a = np.copy(log)
+            a = np.copy(log_arr)
 
         tops, values = utils.tops_from_loglike(a)
 
@@ -1175,8 +1219,10 @@ class Striplog(object):
                basis=None,
                field=None,
                field_function=None,
-               dtype=None,
+               bins=True,
+               dtype='float',
                table=None,
+               sort_table=False,
                legend=None,
                legend_field=None,
                match_only=None,
@@ -1200,6 +1246,12 @@ class Striplog(object):
             field_function (function): Provide a function to apply to the field
                 you are asking for. It's up to you to make sure the function
                 does what you want.
+            bins (bool): Whether to return the index of the items from the
+                lookup table. If False, then the item itself will be returned. 
+            dtype (str): The NumPy dtype string for the output log.
+            table (list): Provide a look-up table of values if you want. If you
+                don't, then it will be constructed from the data.
+            sort_table (bool): Whether to sort the table or not. Default: False.
             legend (Legend): If you want the codes to come from a legend,
                 provide one. Otherwise the codes come from the log, using
                 integers in the order of prevalence. If you use a legend,
@@ -1229,8 +1281,8 @@ class Striplog(object):
             start, stop = basis[0], basis[-1]
             step = basis[1] - start
         else:
-            start = start or self.start.z
-            stop = stop or self.stop.z
+            start = self.start.z if start is None else start
+            stop = self.stop.z if stop is None else stop
             pts = np.ceil((stop - start)/step) + 1
             basis = np.linspace(start, stop, int(pts))
 
@@ -1275,11 +1327,17 @@ class Striplog(object):
         else:
             match_only = []
 
+        if sort_table:
+            table.sort()
+
         start_ix = self.read_at(start, index=True)
         stop_ix = self.read_at(stop, index=True)
         if stop_ix is not None:
             stop_ix += 1
 
+        if self.order == 'elevation':
+            start_ix, stop_ix = stop_ix, start_ix
+
         # Assign the values.
         for i in self[start_ix:stop_ix]:
             c = i.primary
@@ -1297,7 +1355,12 @@ class Striplog(object):
                 f = field_function or utils.null
                 try:
                     v = f(i.data.get(field, undefined)) or undefined
-                    key = (table.index(v) + 1) or undefined
+                    if bins:
+                        # Then return the bin we're in...
+                        key = (table.index(v) + 1) or undefined
+                    else:
+                        # ...else return the actual value.
+                        key = v
                 except ValueError:
                     key = undefined
             else:  # Use the lookup table.
@@ -1309,6 +1372,10 @@ class Striplog(object):
             top_index = int(np.ceil((max(start, i.top.z)-start)/step))
             base_index = int(np.ceil((min(stop, i.base.z)-start)/step))
 
+            # Deal with bug where elevation order seems to be an issue.
+            if self.order == 'elevation':
+                top_index, base_index = base_index, top_index
+
             try:
                 result[top_index:base_index+1] = key
             except:  # Have a list or array or something.
@@ -1533,7 +1600,7 @@ class Striplog(object):
         Args:
             legend (Legend): The Legend to use for colours, etc.
             width (int): The width of the plot, in inches. Default 1.
-            ladder (bool): Whether to use widths or not. Default False.
+            ladder (bool): Whether to use widths or not. Default `True`.
             aspect (int): The aspect ratio of the plot. Default 10.
             ticks (int or tuple): The (minor,major) tick interval for depth.
                 Only the major interval is labeled. Default (1,10).
@@ -1545,6 +1612,14 @@ class Striplog(object):
                 object. Default False.
             colour (str): Which data field to use for colours.
             cmap (cmap): Matplotlib colourmap. Default ``viridis``.
+            default (float): The default (null) value.
+            style (str): Can be 'tops', 'points' or 'field' for different types
+                of plot. If you use 'field', give the name of the field as the
+                `field` argument.
+            field (str): The name of the field to use for the width parameter.
+            label (str): Can be the name of a single Component attribute,
+                or a format string with multiple attributes, like
+                `"{colour} {lithology}"`.
             **kwargs are passed through to matplotlib's ``patches.Rectangle``.
 
         Returns:
@@ -1602,7 +1677,17 @@ class Striplog(object):
 
         if label is not None:
             for iv in self.__list:
-                plt.text(1.6, iv.middle, iv.primary[label], ha='left', va='center', size=10)
+                if '{' in label:
+                    label_text = iv.summary(fmt=label)
+                else:
+                    label_text = iv.primary[label]
+                plt.text(1.6,
+                         iv.middle,
+                         label_text,
+                         ha='left',
+                         va='center',
+                         size=10
+                         )
 
         # Make sure ticks is a tuple.
         try:
@@ -1703,9 +1788,17 @@ class Striplog(object):
             function (function). A function that takes an array as the only
                 input, and returns whatever you want to store in the 'name'
                 attribute of the primary component.
+
         Returns:
-            None. The function works on the striplog in place.
+            A copy of the striplog.
         """
+        # Try to deal with the log potentially being a welly.Curve, pd.Series,
+        # or pd.DataFrame. Squeeze down to 1D in case it's a column vector.
+        try:
+            log_arr = np.squeeze(log.values)
+        except AttributeError:
+            log_arr = np.squeeze(np.asanyarray(log))
+
         # Build a dict of {index: [log values]} to keep track.
         intervals = {}
         previous_ix = -1
@@ -1714,18 +1807,19 @@ class Striplog(object):
             if ix is None:
                 continue
             if ix == previous_ix:
-                intervals[ix].append(log[i])
+                intervals[ix].append(log_arr[i])
             else:
-                intervals[ix] = [log[i]]
+                intervals[ix] = [log_arr[i]]
             previous_ix = ix
 
         # Set the requested attribute in the primary comp of each interval.
+        new_strip = self.copy()
         for ix, data in intervals.items():
             f = function or utils.null
             d = f(np.array(data))
-            self[ix].data[name] = d
+            new_strip[ix].data[name] = d
 
-        return None
+        return new_strip
 
     def find(self, search_term, index=False):
         """
@@ -1741,6 +1835,7 @@ class Striplog(object):
             search_term (string or Component): The thing you want to search
                 for. Strings are treated as regular expressions.
             index (bool): Whether to return the index instead of the interval.
+
         Returns:
             Striplog: A striplog that contains only the 'hit' Intervals.
                 However, if ``index`` was ``True``, then that's what you get.
@@ -2569,3 +2664,78 @@ class Striplog(object):
             proc = proc | log
 
         return Striplog.from_log(proc, components=comps, basis=basis)
+
+    @classmethod
+    def from_macrostrat(cls, lng, lat, buffer_size=0.2):
+        """
+        Create a striplog from components derived using the MacroStrat API.
+            This is simply a helper function to make things easier, but it
+            works because we know what our data looks like in advance.
+
+        Note: In order to plot this, you will need to add space for text and 
+            other decoration. This simply gives a Striplog back which _can_
+            be plotted.
+
+        Args:
+            lng (float): The longitude of the site.
+            lat (float): The latitude of the site.
+            buffer_size (float): The size of the buffer to use, in degrees.
+
+        Returns:
+            Striplog. A new striplog instance.
+
+        Example:
+            >>> lng = -64.3573186
+            >>> lat = 44.4454632
+            >>> buffer_size = 0.3
+            >>> striplog.striplog.from_macrostrat(lng, lat, buffer_size)
+            {'top': Position({'middle': 358.9, 'units': 'm'}), 
+                'base': Position({'middle': 419.2, 'units': 'm'}), 
+                'description': '', 'data': {}, 'components': [Component({
+                    'map_id': 948660.0, 'scale': 'small', 'source_id': 7.0,
+                    'name': 'Devonian plutonic: undivided granitic rocks',
+                    'age': 'devonian', 'lith': 'plutonic: undivided granitic rocks',
+                    'best_age_top': 358.9, 'best_age_bottom': 419.2, 't_int': 94.0,
+                    'b_int': 94.0, 'color': '#cb8c37', 'source': 'MacroStrat.org (CC-BY)})]}
+            {'top': Position({'middle': 358.9, 'units': 'm'}),
+                'base': Position({'middle': 541.0, 'units': 'm'}),
+                'description': '', 'data': {}, 'components': [Component({
+                    'map_id': 948228.0, 'scale': 'small', 'source_id': 7.0,
+                    'name': 'Cambrian-Devonian sedimentary', 'age': 'cambrian-devonian',
+                    'lith': 'sedimentary', 'best_age_top': 358.9, 'best_age_bottom': 541.0,
+                    't_int': 94.0, 'b_int': 122.0, 'color': '#99c08d',
+                    'source': 'MacroStrat.org (CC-BY)})]}
+            {'top': Position({'middle': 443.8, 'units': 'm'}),
+                'base': Position({'middle': 541.0, 'units': 'm'}),
+                'description': '', 'data': {}, 'components': [Component({
+                    'map_id': 973359.0, 'scale': 'small', 'source_id': 7.0,
+                    'name': 'Cambrian-Ordovician sedimentary', 'age': 'cambrian-ordovician',
+                    'lith': 'sedimentary', 'best_age_top': 443.8, 'best_age_bottom': 541.0,
+                    't_int': 112.0, 'b_int': 122.0, 'color': '#409963',
+                    'source': 'MacroStrat.org (CC-BY)})]}
+        """
+        # Get the 
+        features = utils.geology_from_macrostrat(lng=lng, lat=lat,
+                                                 buffer_size=buffer_size)
+
+        columns = ('color', 'lith', 'age')
+
+        intervals = []
+
+        for feature in features:
+            if feature['geometry'] is None:
+                continue
+
+            components = []
+            for lith in utils.get_liths_from_macrostrat(feature['properties']['lith']):
+                c = Component({'lithology': lith})
+                components.append(c)
+
+            intervals.append(Interval(
+                            top=feature['properties']['best_age_top'],
+                            base=feature['properties']['best_age_bottom'],
+                            components=components,
+                            description=feature['properties']['descrip'])
+                            )
+
+        return cls(intervals, source='Macrostrat [CC-BY]', order='age')
